#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Apr 11 10:27:23 2018

@author: aguimera
"""
import numpy as np
from scipy import signal
from fractions import Fraction


def RemoveDC(sig):
    st = np.array(sig)
    st = signal.detrend(st, axis=0)
    return sig.duplicate_with_new_array(signal=st*sig.units)


def Gain(sig, Gain):
    return sig*Gain


def Resample(sig, Fs=None, MaxPoints=None):
    if MaxPoints is None:
        f = Fs/sig.sampling_rate
        fact = Fraction(float(f)).limit_denominator()
        dowrate = fact.denominator
        uprate = fact.numerator
    else:
        dowrate = int(sig.times.shape[0]/MaxPoints)
        if dowrate > 0:
            f = float(1/float(dowrate))
            uprate = 1

    if dowrate > 0:
        print sig.sampling_rate*f, f, uprate, dowrate
        rs = signal.resample_poly(np.array(sig), uprate, dowrate)
        sig = sig.duplicate_with_new_array(signal=rs*sig.units)
        sig.sampling_rate = sig.sampling_rate*f
        return sig
    else:
        return sig


def Filter(sig, Type, Order, Freqs):
    st = np.array(sig)
    freqs = Freqs/(0.5*sig.sampling_rate)

    b, a = signal.butter(Order, freqs, Type)
    st = signal.filtfilt(b, a, st, axis=0)

    return sig.duplicate_with_new_array(signal=st*sig.units)


class SignalProcess():
    def __init__(self, Signal, ProcessChain):
        self.Signal = Signal
        self.signal = Signal.signal
        self.ProcessChain = ProcessChain
        self.Name = Signal.Name

    def GetSignal(self, Time, Units=None):
        sig = self.Signal.GetSignal(Time, Units)
        for Proc in self.ProcessChain:
            sig = Proc['function'](sig, **Proc['args'])
        return sig



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Apr 30 00:22:59 2018

@author: aguimera
"""
import PhyREC.SignalProcess as RPro
import types
import inspect

Functions = [RPro.__dict__.get(a) for a in dir(RPro)
             if isinstance(RPro.__dict__.get(a), types.FunctionType)]


for f in Functions:
    print f.__code__.co_name
    print f.__code__.co_argcount
    for arg in inspect.getargspec(f)[0]:
        print arg

    


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Apr 11 11:12:23 2018

@author: aguimera
"""
import numpy as np
import matplotlib.pyplot as plt
import quantities as pq
from scipy import signal
from collections import OrderedDict


def threshold_detection(signal, threshold=0.0 * pq.mV, sign='above',
                        RelaxTime=None):
    """
    Returns the times when the analog signal crosses a threshold.
    Usually used for extracting spike times from a membrane potential.
    Adapted from version in NeuroTools.

    Parameters
    ----------
    signal : neo AnalogSignal object
        'signal' is an analog signal.
    threshold : A quantity, e.g. in mV
        'threshold' contains a value that must be reached
        for an event to be detected. Default: 0.0 * mV.
    sign : 'above' or 'below'
        'sign' determines whether to count thresholding crossings
        that cross above or below the threshold.
    format : None or 'raw'
        Whether to return as SpikeTrain (None)
        or as a plain array of times ('raw').

    Returns
    -------
    result_st : neo SpikeTrain object
        'result_st' contains the spike times of each of the events (spikes)
        extracted from the signal.
    """

    assert threshold is not None, "A threshold must be provided"

    if sign == 'above':
        cutout = np.where(signal > threshold)[0]
    elif sign == 'below':
        cutout = np.where(signal < threshold)[0]

    if len(cutout) <= 0:
        events = np.zeros(0)
    else:
        take = np.where(np.diff(cutout) > 1)[0] + 1
        take = np.append(0, take)

        time = signal.times
        events = time[cutout][take]

    if RelaxTime:
        outevents = []
        told = 0*pq.s
        for te in events:
            if (te-told) > RelaxTime:
                outevents.append(te)
                told = te
    else:
        outevents = events

    return outevents


def PlotPSD(Signals, Time=None, nFFT=2**17, FMin=None, Ax=None,
            scaling='density', Units=None):

    if Ax is None:
        Fig, Ax = plt.subplots()

    PSD = {}        
    for sl in Signals:
        if not hasattr(sl, 'GetSignal'):
            continue
        sig = sl.GetSignal(Time, Units=Units)
        if FMin is not None:
            nFFT = int(2**(np.around(np.log2(sig.sampling_rate.magnitude/FMin))+1))

        ff, psd = signal.welch(x=sig, fs=sig.sampling_rate, axis=0,
                               window='hanning',
                               nperseg=nFFT,
                               scaling=scaling)
        if scaling == 'density':
            units = sig.units**2/pq.Hz
        elif scaling == 'spectrum':
            units = sig.units**2

        slN = sl.Name
        PSD[slN] = {}
        PSD[slN]['psd'] = psd * units
        PSD[slN]['ff'] = ff

        if hasattr(sl, 'Line'):
            Line = sl.Line
        else:
            Line = '-'
        if hasattr(sl, 'Color'):
            Color = sl.Color
        else:
            Color = None
        if hasattr(sl, 'Alpha'):
            Alpha = sl.Alpha
        else:
            Alpha = 1
        if hasattr(sl, 'DispName'):
            label = sl.DispName
        else:
            label = sl.Name

        Ax.loglog(ff, psd,
                  Line,
                  color=Color,
                  label=label,
                  alpha=Alpha)

    Ax.set_xlabel('Frequency [Hz]')
    Ax.set_ylabel('[' + str(units).split(' ')[-1] + ']')

    handles, labels = Ax.get_legend_handles_labels()
    by_label = OrderedDict(zip(labels, handles))

    nLines = len(by_label)
    nlc = 4
    if nLines > nlc:
        ncol = (nLines / nlc) + ((nLines % nlc) > 0)
    else:
        ncol = 1
    Ax.legend(by_label.values(), by_label.keys(),
              loc='best',
              ncol=ncol,
              fontsize='x-small')

    return PSD


def PlotEventAvg(Signals, TimesEvent, TimeAvg, Time=None,
                 OverLap=True, Std=True, Units=None, FileOutPrefix=None):

    ft, Axt = plt.subplots()

    for sl in Signals:
        avg = np.array([])
        fig, Ax = plt.subplots()

        Ts = sl.signal.sampling_period
        nSamps = int((TimeAvg[1]-TimeAvg[0])/Ts)
        t = np.arange(nSamps)*Ts + TimeAvg[0]

        for et in TimesEvent:
            start = et+TimeAvg[0]
            stop = et+TimeAvg[1]

            st = sl.GetSignal((start, stop), Units=Units)[:nSamps]
            try:
                avg = np.hstack([avg, st]) if avg.size else st
                if OverLap:
                    Ax.plot(t, st, 'k-', alpha=0.1)
            except:
                print 'Error', nSamps, et, avg.shape, st.shape

        MeanT = np.mean(avg, axis=1)
        Ax.plot(t, MeanT, 'r-')
        if Std:
            StdT = np.std(avg, axis=1)
            Ax.fill_between(t, MeanT+StdT, MeanT-StdT,
                            facecolor='r', alpha=0.5)

        ylim = Ax.get_ylim()
        Ax.vlines(0, ylim[0], ylim[1],
                  linestyles='dashdot',
                  alpha=0.7,
                  colors='g')
        Ax.ticklabel_format(axis='y', style='sci', scilimits=(-2, 2))
        plt.title(sl.Name)
        Ax.set_xlabel('Time [s]')
        su = str(st.units).split(' ')[-1]
        Ax.set_ylabel(sl.Name + ' [' + su + ']')

        Axt.plot(t, MeanT, label=sl.Name)
        if FileOutPrefix is not None:
            fig.savefig(FileOutPrefix + sl.Name + '.png')

    ylim = Axt.get_ylim()
    Axt.vlines(0, ylim[0], ylim[1],
               linestyles='dashdot',
               alpha=0.7,
               colors='g')

    Axt.set_ylabel('[' + su + ']')
    Axt.set_xlabel('Time [s]')
    Axt.ticklabel_format(axis='y', style='sci', scilimits=(-2, 2))
    Axt.legend()
    if FileOutPrefix is not None:
        ft.savefig(FileOutPrefix + '.png')


#        if Spect:
#            nFFT = int(2**(np.around(np.log2(Fs/sl.SpecFmin))+1))
#            noverlap = int((Ts*nFFT - sl.SpecTimeRes)/Ts)
#            TWindOff = (nFFT * Ts)/8
#
#            f, tsp, Sxx = signal.spectrogram(MeanT, Fs,
#                                             window='hanning',
#                                             nperseg=nFFT,
#                                             noverlap=noverlap,
#                                             scaling='density',
#                                             axis=0)
#
#            finds = np.where(f < sl.SpecFmax)[0][1:]
#            print Sxx.shape
#            r, c = Sxx.shape
#            S = Sxx.reshape((r, c))[finds][:]
#            pcol = AxS.pcolormesh(tsp + TimeWindow[0].magnitude + TWindOff,
#                                  f[finds],
#                                  np.log10(S),
#                                  vmin=np.log10(np.max(S))+sl.SpecMinPSD,
#                                  vmax=np.log10(np.max(S)),
#                                  cmap=sl.SpecCmap)
#            f, a = plt.subplots(1, 1)
#            f.colorbar(pcol)
#
#        Axt.plot(t, np.mean(avg, axis=1), label=sl.DispName)
#        ft.canvas.draw()
#
#    Axt.ticklabel_format(axis='y', style='sci', scilimits=(-2, 2))
#    Axt.legend()
#    ft.canvas.draw()
#    plt.show()

#
#    def PlotHist(self, Time, Resamp=False, Binds=250):
#        fig, Ax = plt.subplots()
#
#        for sl in self.Slots:
#            Ax.hist(sl.GetSignal(Time, Resamp=Resamp),
#                    Binds,
#                    alpha=0.5)
#            Ax.set_yscale('log')
#
#    def PlotPSD(self, Time, nFFT=2**17, FMin=None, Resamp=False):
#
#        if not self.FigFFT or not plt.fignum_exists(self.FigFFT.number):
#            self.FigFFT, self.AxFFT = plt.subplots()
#
#        PSD = {}
#        for sl in self.Slots:
#            sig = sl.GetSignal(Time, Resamp=Resamp)
#            if FMin:
#                nFFT = int(2**(np.around(np.log2(sig.sampling_rate.magnitude/FMin))+1)) 
#
#            ff, psd = signal.welch(x=sig, fs=sig.sampling_rate,
#                                   window='hanning',
#                                   nperseg=nFFT, scaling='density', axis=0)
#            slN = sl.SigName
#            PSD[slN] = {}
#            PSD[slN]['psd'] = psd
#            PSD[slN]['ff'] = ff
#            self.AxFFT.loglog(ff, psd, label=sl.DispName)
#
#        self.AxFFT.set_xlabel('Frequency [Hz]')
#        self.AxFFT.set_ylabel('PSD [V^2/Hz]')
#        self.AxFFT.legend()
#        return PSD



#
#    def PlotPSDSNR(self, (EventRec, EventName), TDelay, TEval, DevDCVals,
#                   Time=None, nFFT=2**17):
#
#        etimes = EventRec.GetEventTimes(EventName, Time)
#
#        DevACVals = {}
#
#        for sl in self.Slots:
#            fig, (ax1, ax2)  = plt.subplots(1,2)
#            psd = np.array([])
#            DCVals = DevDCVals[sl.SigName]
#            for ne, et in enumerate(etimes):
#                start = et+TDelay
#                stop = et+TDelay+TEval
#
#                sig = sl.GetSignal((start, stop), Resamp=False)
#                fpsd, npsd = signal.welch(x=sig, fs=sig.sampling_rate,
#                                          window='hanning',
#                                          nperseg=nFFT, scaling='density', axis=0)                
#                psd = np.hstack([psd, npsd]) if psd.size else npsd
#
##                Flin = fpsd[1:].magnitude
##                Flog = np.logspace(np.log10(Flin[0]),
##                                   np.log10(Flin[-1]), 100)
##                Flog = np.round(Flog, 9)
##                Flin = np.round(Flin, 9)
##                intpsd = interpolate.interp1d(Flin, npsd[1:].transpose())(Flog)
##                a, b, _ = FETAna.noise.FitNoise(Flog,
##                                                intpsd, Fmin=150, Fmax=5e3)
#
#                ax1.loglog(fpsd, npsd)
##                ax2.loglog(Flog, intpsd/FETAna.noise(Flog, a, b))
#
#            psdD = {'Vd0': psd.transpose()}
#            ACVals = {'PSD': psdD,
#                      'gm': None,
#                      'Vgs': DCVals['Vgs'],
#                      'Vds': DCVals['Vds'],
#                      'Fpsd': fpsd.magnitude,
#                      'Fgm': None,
#                      'ChName': sl.SigName,
#                      'Name': sl.DispName,
#                      'GMPoly': DCVals['GMPoly'],
#                      'IdsPoly': DCVals['IdsPoly'],
#                      'Ud0': DCVals['Ud0'],
#                      'IsOK': DCVals['IsOK'],
#                      'DateTime': DCVals['DateTime']}
#            DevACVals[sl.DispName] = ACVals
#            fig.canvas.draw()
#            plt.show()
#
#        FETAna.InterpolatePSD(DevACVals)
#        FETAna.FitACNoise(DevACVals, Fmin=150, Fmax=5e3, IsOkFilt=False)
#        pltPSD = FETplt.PyFETPlot()
#        pltPSD.AddAxes(('PSD', 'NoA', 'NoB'))
#        pltPSD.AddLegend()
#        pltPSD.PlotDataCh(DevACVals)
#
#        pickle.dump(DevACVals, open('test.pkl', 'wb'))
#
#        return DevACVals



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Jul 26 12:11:53 2017

@author: aguimera
"""

import numpy as np
import matplotlib.pyplot as plt
import quantities as pq
from scipy import signal
import matplotlib.colors as colors
from collections import OrderedDict
from scipy.interpolate import interp2d


class SpecSlot():
    def __init__(self, Signal, Units=None, Position=None, DispName=None,):
        self.Fres = 5.0
        self.TimeRes = 0.2
        self.Fmin = 1
        self.Fmax = 150
        self.MinPSD = None
        self.MaxPSD = None
        self.MaxPSDrange = None
        self.LogNormalize = True
        self.Cmap = 'jet'
        self.LogScale = False

        self.Ax = None
        self.CAx = None
        self.Fig = None

        self.Signal = Signal

        self.Units = Units
        self.Position = Position
        if DispName is None:
            self.DispName = self.Signal.Name
        else:
            self.DispName = DispName

    def PlotSignal(self, Time, Units=None):
        if Units is not None:
            self.Units = Units
        sig = self.Signal.GetSignal(Time, Units=self.Units)

        nFFT = int(2**(np.around(np.log2(sig.sampling_rate.magnitude/self.Fres))+1))
        Ts = sig.sampling_period.magnitude
        noverlap = int((Ts*nFFT - self.TimeRes)/Ts)

        f, t, Sxx = signal.spectrogram(sig, sig.sampling_rate,
                                       window='hanning',
                                       nperseg=nFFT,
                                       noverlap=noverlap,
                                       scaling='spectrum',
                                       axis=0)

        finds = np.where((self.Fmin < f) & (f < self.Fmax))[0][1:]
        r, g, c = Sxx.shape
        data = Sxx.reshape((r, c))[finds][:]

        if self.MaxPSD is None:
            MaxPSD = data.max()
        else:
            MaxPSD = self.MaxPSD

        if self.MinPSD is None:
            if self.MaxPSDrange is None:
                MinPSD = 10**(np.log10(MaxPSD)-5)
            else:
                MinPSD = 10**(np.log10(MaxPSD)-self.MaxPSDrange)
        else:
            MinPSD = self.MinPSD

        if self.LogNormalize:
            Norm = colors.LogNorm(MinPSD, MaxPSD)
        else:
            Norm = colors.Normalize(MinPSD, MaxPSD)

        if self.Ax is None:
            self.Fig, self.Ax = plt.subplots()

        x = t + sig.t_start.magnitude
        y = f[finds].magnitude
        img = self.Ax.imshow(data,
                             cmap='jet',
                             norm=Norm,
                             interpolation='quadric',
                             origin='lower',
                             aspect='auto',
                             extent=(np.min(x), np.max(x), np.min(y), np.max(y)))

        cbar = plt.colorbar(img, ax=self.CAx, fraction=0.8)
        cbar.ax.tick_params(length=1, labelsize='x-small')

        su = str(sig.units).split(' ')[-1]
        label = "[{}^2]".format(su)
        cbar.set_label(label, fontsize='x-small')

        if self.LogScale:
            self.Ax.set_yscale('log')


class WaveSlot():
    UnitsInLabel = True

    def __init__(self, Signal, Units=None, Position=None, DispName=None,
                 Color='k', Line='-', Alpha=1, Ylim=None, LineWidth=0.5):
        self.Signal = Signal

        self.Units = Units
        self.Position = Position
        if DispName is None:
            self.DispName = self.Signal.Name
        else:
            self.DispName = DispName

        self.Ax = None
        self.Fig = None

        self.Color = Color
        self.Line = Line
        self.Alpha = Alpha
        self.Ylim = Ylim
        self.LineWidth = LineWidth

        self.Name = self.Signal.Name

    def GetSignal(self, Time, Units=None):
        if Units is not None:
            self.Units = Units

        sig = self.Signal.GetSignal(Time, self.Units)
        self.units = sig.units
        return sig

    def PlotSignal(self, Time, Units=None):
        if self.Ax is None:
            self.Fig, self.Ax = plt.subplots()
        if Units is not None:
            self.Units = Units

        sig = self.Signal.GetSignal(Time, self.Units)

        if self.UnitsInLabel is True:
            su = str(sig.units).split(' ')[-1]
            label = "{} [{}]".format(self.DispName, su)
        else:
            label = self.DispName

        self.Ax.plot(sig.times,
                     sig,
                     self.Line,
                     linewidth=self.LineWidth,
                     color=self.Color,
                     label=label,
                     alpha=self.Alpha)

        if self.Ylim is not None:
            self.Ax.set_ylim(self.Ylim)


class PlotSlots():
    LegNlabCol = 4  # Number of labels per col in legend
    LegFontSize = 'xx-small'

    def __init__(self, Slots, ShowNameOn='Axis', figsize=None,
                 ShowAxis=True, AutoScale=True):
        self.ShowNameOn = ShowNameOn  # 'Axis', 'Legend', None
        self.Slots = Slots
        self.ShowAxis = ShowAxis
        self.AutoScale = AutoScale

        Pos = []
        for isl, sl in enumerate(self.Slots):
            if sl.Position is None:
                sl.Position = isl
            Pos.append(sl.Position)

        self.Fig, A = plt.subplots(max(Pos) + 1, 2,
                                   sharex=True,
                                   figsize=figsize,
                                   gridspec_kw={'width_ratios': (10, 1)})
        self.Axs = [a[0] for a in A]
        self.CAxs = [a[1] for a in A]

        for ca in self.CAxs:
            ca.axis('off')

        for sl in self.Slots:
            if isinstance(sl, SpecSlot):
                sl.CAx = self.CAxs[sl.Position]
            sl.Ax = self.Axs[sl.Position]
            sl.Fig = self.Fig

        self.SlotsInAxs = {}
        for ax in self.Axs:
            sll = []
            for sl in self.Slots:
                if sl.Ax == ax:
                    sll.append(sl)
            self.SlotsInAxs.update({ax: sll})

    def ClearAxes(self):
        for sl in self.Slots:
            while sl.Ax.lines:
                sl.Ax.lines[0].remove()

    def FormatFigure(self):
        if self.ShowAxis:
            for Ax in self.Axs:
    #            Ax.get_yaxis().set_visible(False)
                Ax.get_xaxis().set_visible(False)
                Ax.spines['top'].set_visible(False)
                Ax.spines['right'].set_visible(False)
    #            Ax.spines['left'].set_visible(False)
                Ax.spines['bottom'].set_visible(False)
                Ax.ticklabel_format(axis='y', style='sci', scilimits=(-3, 3))

            TimeAx = self.Axs[-1]
            TimeAx.set_xlabel('Time [s]')
            TimeAx.get_xaxis().set_visible(True)
        else:
            for Ax in self.Axs:
                Ax.get_yaxis().set_visible(False)
                Ax.get_xaxis().set_visible(False)
                Ax.spines['top'].set_visible(False)
                Ax.spines['right'].set_visible(False)
                Ax.spines['left'].set_visible(False)
                Ax.spines['bottom'].set_visible(False)

        self.Fig.subplots_adjust(top=1.0,
                                 bottom=0.05,
                                 left=0.1,
                                 right=0.94,
                                 hspace=0.0,
                                 wspace=0.0)
#        self.Fig.tight_layout()

    def AddLegend(self, Ax):
        if isinstance(self.SlotsInAxs[Ax][0], SpecSlot):
            self.SlotsInAxs[Ax][0].Ax.set_ylabel('Frequency [Hz]',
                                                 fontsize=self.LegFontSize)
        elif self.ShowNameOn is None:
            return
        elif self.ShowNameOn == 'Axis':
            lbls = []
            for sl in self.SlotsInAxs[Ax]:
                su = str(sl.Units).split(' ')[-1]
                lbls.append("{} [{}]".format(sl.DispName, su))
            sl.Ax.set_ylabel('\n'.join(set(lbls)),
                             fontsize=self.LegFontSize)
        elif self.ShowNameOn == 'Legend':
            handles, labels = Ax.get_legend_handles_labels()
            by_label = OrderedDict(zip(labels, handles))

            nLines = len(by_label)
            nlc = self.LegNlabCol
            if nLines > nlc:
                ncol = (nLines / nlc) + ((nLines % nlc) > 0)
            else:
                ncol = 1
            Ax.legend(by_label.values(), by_label.keys(),
                      loc='best',
                      ncol=ncol,
                      fontsize=self.LegFontSize)

    def PlotChannels(self, Time, Units=None):
        self.ClearAxes()
        for sl in self.Slots:
            sl.PlotSignal(Time, Units=Units)
        if Time is not None:
            sl.Ax.set_xlim(Time[0], Time[1])

        for Ax in self.Axs:
            self.AddLegend(Ax)
            if self.AutoScale:
                Ax.autoscale(enable=True, axis='y')

        self.FormatFigure()

    def PlotEvents(self, Times, color='r', alpha=0.5):
        for ax in self.Axs:
            ylim = ax.get_ylim()
            ax.vlines(Times, ylim[0], ylim[1], color=color, alpha=alpha)
        



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Jul 26 12:11:53 2017

@author: aguimera
"""

import neo
import numpy as np
import quantities as pq


class NeoSegment():
    def __init__(self, RecordFile=None, Seg=None):
        self.SigNames = {}
        self.EventNames = {}
        if RecordFile is None:
            if Seg is None:
                self.Seg = neo.Segment('New Seg')
            else:
                self.Seg = Seg
                self.UpdateEventDict()
                self.UpdateSignalsDict()
            return

        ftype = RecordFile.split('.')[-1]
        if ftype == 'h5':
            self.RecFile = neo.io.NixIO(filename=RecordFile, mode='ro')
            Block = self.RecFile.read_block()
        elif ftype == 'smr':
            self.RecFile = neo.io.Spike2IO(filename=RecordFile)
            Block = self.RecFile.read()[0]

        self.Seg = Block.segments[0]

        self.UpdateSignalsDict()
        self.UpdateEventDict()

    def SaveRecord(self, FileName):
        if FileName.endswith('.h5'):
            out_f = neo.io.NixIO(filename=FileName)
        elif FileName.endswith('.mat'):
            out_f = neo.io.NeoMatlabIO(filename=FileName)
        else:
            return

        out_bl = neo.Block(name='NewBlock')
        out_bl.segments.append(self.Seg)
        out_f.write_block(out_bl)
        if FileName.endswith('.h5'):
            out_f.close()

    def UpdateSignalsDict(self):
        self.SigNames = {}
        for i, sig in enumerate(self.Seg.analogsignals):
            if sig.name is None:
                name = str(i)
                if sig.annotations is not None:
                    if 'nix_name' in sig.annotations.keys():
                        name = sig.annotations['nix_name']
            else:
                name = str(sig.name)

            self.SigNames.update({name: i})

    def UpdateEventDict(self):
        self.EventNames = {}
        for i, eve in enumerate(self.Seg.events):
            if eve.name is None:
                try:
                    name = eve.annotations['title']
                except:
                    print 'Event found no name ', i
                    name = str(i)
                self.EventNames.update({name: i})
            else:
                self.EventNames.update({eve.name: i})

    def GetEventTimes(self, EventName, Time=None):
        eve = self.Seg.events[self.EventNames[EventName]].times
        if Time:
            events = eve[np.where((eve > Time[0]) & (eve < Time[1]))]
        else:
            events = eve
        return events

    def GetTstart(self, ChName):
        return self.Seg.analogsignals[self.SigNames[ChName]].t_start

    def SetTstart(self, ChName, Tstart):
        self.Seg.analogsignals[self.SigNames[ChName]].t_start = Tstart

    def SetSignal(self, ChName, Sig):
        self.Seg.analogsignals[self.SigNames[ChName]] = Sig

    def GetSignal(self, ChName):
        return self.Seg.analogsignals[self.SigNames[ChName]]

    def AddEvent(self, Times, Name):
        eve = neo.Event(times=Times,
                        units=pq.s,
                        name=Name)

        self.Seg.events.append(eve)
        self.UpdateEventDict()

    def AddSignal(self, Sig):
        self.Seg.analogsignals.append(Sig)
        self.UpdateSignalsDict()

    def AppendSignal(self, ChName, Vect):
        sig = self.Seg.analogsignals[self.SigNames[ChName]]

        v_old = np.array(sig)
        v_new = np.vstack((v_old, Vect))

        sig_new = sig.duplicate_with_new_array(signal=v_new*sig.units)

        self.SetSignal(ChName, sig_new)


class NeoSignal():
    def __init__(self, NeoSeg=None, SigName=None, Signal=None):
        if Signal is None:
            self.Signal = NeoSeg.GetSignal(SigName)
            self.signal = NeoSeg.GetSignal(SigName)
        else:
            self.Signal = Signal
            self.signal = Signal
        self.Name = self.Signal.name

    def GetSignal(self, Time, Units=None):
        time = self.CheckTime(Time)

        sl = self.Signal.time_slice(time[0], time[1])

        if Units is None:
            return sl
        else:
            return sl.rescale(Units)

    def CheckTime(self, Time):
        if Time is None:
            return (self.Signal.t_start, self.Signal.t_stop)

        if Time[0] < self.Signal.t_start:
            Tstart = self.Signal.t_start
        else:
            Tstart = Time[0]

        if Time[1] > self.Signal.t_stop:
            Tstop = self.Signal.t_stop
        else:
            Tstop = Time[1]

        return (Tstart, Tstop)




#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Thu Dec 21 18:30:42 2017

@author: aguimera
"""

#  Copyright 2017 Anton Guimerà Brunet <anton.guimera@csic.es>
#
#  This file is part of PyGFET.
#
#  PyGFET is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  PyGFET is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.


from setuptools import setup, find_packages

_version = '0.0.1b1'

long_description = """
                   Library for electrphysiological analysis based on neo                   
                   """

install_requires = ['numpy',
                    'matplotlib',
                    'quantities>=0.12',
                    'scipy',
                    'neo>=0.5.2',
                    'nixio']

console_scripts = ['PhyRecExplorer = PhyREC.GuiRecordView.GuiRecordExplorer:main',
                   ]

entry_points = {'console_scripts': console_scripts, }

classifiers = ['Development Status :: 3 - Alpha',
               'Environment :: Console',
               'Environment :: X11 Applications :: Qt',
               'Environment :: Win32 (MS Windows)',
               'Intended Audience :: Science/Research',
               'License :: OSI Approved :: GNU General Public License (GPL)',
               'Operating System :: Microsoft :: Windows',
               'Operating System :: POSIX',
               'Operating System :: POSIX :: Linux',
               'Operating System :: Unix',
               'Operating System :: OS Independent',
               'Programming Language :: Python',
               'Programming Language :: Python :: 2.7',
               'Topic :: Scientific/Engineering',
               'Topic :: Software Development :: User Interfaces']

setup(name="PhyREC",
      version=_version,
      description="Library for electrphysiological analysis based on neo",
      long_description=long_description,
      author="Anton Guimerà-Brunet",
      author_email="anton.guimera@csic.es",
      maintainer="Anton Guimerà-Brunet",
      maintainer_email="anton.guimera@csic.es",
      url="https://github.com/aguimera/PhyREC",
      download_url="https://github.com/aguimera/PhyREC",
      license="GPLv3",
      packages=find_packages(),
      classifiers=classifiers,
      entry_points=entry_points,
      install_requires=install_requires,
      include_package_data=True,
      )



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Tue Mar  7 18:09:07 2017

@author: aguimera
"""

import os
import neo
import numpy as np
import matplotlib.pyplot as plt
import quantities as pq
import sys
from qtpy.QtWidgets import QHeaderView
from qtpy import QtWidgets, uic
from qtpy.QtCore import Qt
from qtpy.QtWidgets import QFileDialog, QColorDialog, QInputDialog, QAction
from PyREC.RecordCore import NeoRecord
from PyREC.RecordPlot import PltSlot, PlotRecord, threshold_detection


class RecordExplorer(QtWidgets.QMainWindow):
    OutFigFormats = ('svg', 'png')
    DevDCVals = None
    # 'SlotProperty' (Header, position, string Conv, Editable)
    ChFields = {'Position': ('Position', 0, 1, 1),
                'DispName': ('DispName', 1, 0, 1),
                'SigName': ('SigName', 2, 0, 0),
                'PlotType': ('PlotType', 3, 0, 0),
                'TStart': ('TStart', 4, 2, 1),
                'OutType': ('Out', 5, 0, 1),
                'IVGain': ('IVGain', 6, 3, 1),
                'Gm': ('Gm', 7, 3, 1),
                'GmSignal': ('GmSig', 8, 3, 1),
                'FiltType': ('FiltType', 9, 4, 1),
                'FiltOrder': ('FiltOrder', 10, 5, 1),
                'FiltF1': ('FiltF1', 11, 5, 1),
                'FiltF2': ('FiltF2', 12, 5, 1),
                'RecN': ('RecNumber', 13, 1, 0),
                'SpecFmin': ('SpecFmin', 14, 3, 1),
                'SpecFmax': ('SpecFmax', 15, 3, 1),
                'SpecTimeRes': ('SpecTimeRes', 16, 3, 1),
                'SpecMinPSD': ('SpecMinPSD', 17, 3, 1),
                'SpecCmap': ('SpecCmap', 18, 0, 1),
                'Ymax': ('Y Max', 19, 3, 1),
                'Ymin': ('Y min', 20, 3, 1),
                'FileName': ('FileName', 21, 0, 0)}

    def InitMenu(self):
        mainMenu = self.menuBar
        fileMenu = mainMenu.addMenu('File')

        SaveRec1Action = QAction('Save Rec1 as', self)
        SaveRec1Action.setStatusTip('Save a copy of Rec1 and Evens')
        SaveRec1Action.triggered.connect(self.SaveRec1)
        fileMenu.addAction(SaveRec1Action)

        SaveJoinRecAction = QAction('Join Rec1 and Rec2 as', self)
        SaveJoinRecAction.setStatusTip('Save a copy of Rec1 and Rec2 with Tstart')
        SaveJoinRecAction.triggered.connect(self.SaveJoinRec)
        fileMenu.addAction(SaveJoinRecAction)

        SaveFigAction = QAction('Save Figures', self)
        SaveFigAction.setShortcut('Ctrl+s')
        SaveFigAction.setStatusTip('Save all open figures')
        SaveFigAction.triggered.connect(self.SaveFigures)
        fileMenu.addAction(SaveFigAction)

        CloseFigsAction = QAction('Close Figures', self)
        CloseFigsAction.setStatusTip('Close all open figures')
        CloseFigsAction.triggered.connect(self.CloseFigures)
        fileMenu.addAction(CloseFigsAction)

    def __init__(self):
        QtWidgets.QMainWindow.__init__(self)
        uipath = os.path.join(os.path.dirname(__file__), 'GuiRecordExplorer.ui')
        uic.loadUi(uipath, self)
        
        self.InitMenu()

        self.setWindowTitle('Record Explorer')

        self.ButFileRec.clicked.connect(self.ButFileRecClick)
        self.ButADDRec.clicked.connect(self.ButADDRecClick)
        self.ButDelete.clicked.connect(self.ButDeleteClick)

        self.ButFileRec2.clicked.connect(self.ButFileRec2Click)
        self.ButADDRec2.clicked.connect(self.ButADDRec2Click)

        self.SpnTStart.valueChanged.connect(self.SpnTStartChanged)
        self.SpnTShow.valueChanged.connect(self.SpnTShowChanged)
        self.SLTStart.valueChanged.connect(self.SLTStartChanged)
        self.SLTStop.valueChanged.connect(self.SLTStopChanged)
        self.ButNextEvent.clicked.connect(self.ButNextEventClick)

        self.ButPSD.clicked.connect(self.ButButPSDClick)
        self.ButEventAvg.clicked.connect(self.ButEventAvgClick)

        self.ButSaveView.clicked.connect(self.ButSaveViewClick)

        self.ButThres.clicked.connect(self.ButThresClick)
        self.ButPSDSNR.clicked.connect(self.ButPSDSNRClick)

        self.ButLoadDC.clicked.connect(self.ButLoadDCClick)
        self.ButViewDC.clicked.connect(self.ButViewDCClick)

        self.ButColor.clicked.connect(self.ButColorClick)

        # Init Tables
        self.TblChs.setColumnCount(len(self.ChFields))
        for c in self.ChFields.values():
            self.TblChs.setHorizontalHeaderItem(c[1],
                                                QtWidgets.QTableWidgetItem(c[0]))
        header = self.TblChs.horizontalHeader()
        header.setSectionResizeMode(QHeaderView.ResizeToContents)

        self.ButPlot.clicked.connect(self.ButPlotClick)

    def CloseFigures(self):
        plt.close('all')

    def SaveFigures(self):
        Dir = QFileDialog.getExistingDirectory(self)
        Prefix, okPressed = QInputDialog.getText(self,
                                                 'Prefix',
                                                 'Prefix for files',
                                                 text='Figure')

        if Dir and okPressed:
            for i in plt.get_fignums():
                plt.figure(i)
                for ext in self.OutFigFormats:
                    fileOut = Dir + '/' + Prefix + '{}.' + ext
                    print fileOut
                    plt.savefig(fileOut.format(i))

    def ButColorClick(self):
        color = QColorDialog.getColor()
        if color.isValid():
            print color.name(), color.rgba64()

    def ButFileRecClick(self):

        RecordFile, _ = QFileDialog.getOpenFileName(self,
                                                    "Recording File", "",
                                                    "NixIO (*.h5);;Spike2 (*.smr)")

        print RecordFile
        if RecordFile:
            Gain = self.SpnRecGain.value()
            self.rec = NeoRecord(RecordFile, UnitGain=Gain)

            self.LblRecFile.setText(RecordFile.split('/')[-1])
            self.UpdateTimeControl()

            self.LstChRec.clear()
            for ch in sorted(self.rec.SigNames.keys()):
                self.LstChRec.addItem(ch)

            self.LstEvents.clear()
            for ch in sorted(self.rec.EventNames.keys()):
                self.LstEvents.addItem(ch)

    def ButFileRec2Click(self):
        RecordFile, _ = QFileDialog.getOpenFileName(self,
                                                    "Recording File", "",
                                                    "NixIO (*.h5);;Spike2 (*.smr)")

        if RecordFile:
            Gain = self.SpnRec2Gain.value()
            self.rec2 = NeoRecord(RecordFile, UnitGain=Gain)

            self.LblRecFile2.setText(RecordFile.split('/')[-1])

            self.LstChRec2.clear()
            for ch in sorted(self.rec2.SigNames.keys()):
                self.LstChRec2.addItem(ch)

            self.LstEvents2.clear()
            for ch in sorted(self.rec2.EventNames.keys()):
                self.LstEvents2.addItem(ch)

    def ButDeleteClick(self):
        try:
            sel = self.TblChs.selectedRanges()
            self.TblChs.removeRow(sel[0].topRow())
        except:
            print 'ERROR deleting row'

    def ButADDRec2Click(self):
        self.TblChs.setSortingEnabled(False)

        chs = []
        for s in self.LstChRec2.selectedItems():
            chs.append(s.text())

        rc = len(chs)
        offset = self.TblChs.rowCount()
        self.TblChs.setRowCount(rc+offset)
        index = -1

        for ir, ch in enumerate(chs):
            index += 1
            Slinf = PltSlot()
            Slinf.TStart = self.SpnTstart2.value()*pq.s
            Slinf.RecN = 2
            Slinf.DispName = ch
            Slinf.SigName = ch
            Slinf.Position = index+offset
            Slinf.FileName = self.rec2.RecFile.filename
            Slinf.PlotType = self.CmbWaveType.currentText()
            Slinf.OutType = 'V'

            self.UpdateRownTableChannels(Slinf,
                                         Row=index+offset)

        self.TblChs.setSortingEnabled(True)

    def ButADDRecClick(self):
        self.TblChs.setSortingEnabled(False)

        chs = []
        for s in self.LstChRec.selectedItems():
            chs.append(s.text())

        rc = len(chs)
        offset = self.TblChs.rowCount()
        self.TblChs.setRowCount(rc+offset)
        index = -1

        for ir, ch in enumerate(chs):
            index += 1
            Slinf = PltSlot()
            Slinf.TStart = self.rec.GetTstart(ch)
            Slinf.RecN = 1
            Slinf.DispName = ch
            Slinf.SigName = ch
            Slinf.Position = index+offset
            Slinf.FileName = self.rec.RecFile.filename
            Slinf.PlotType = self.CmbWaveType.currentText()
            if ch.startswith('T'):
                Slinf.OutType = 'I'
            else:
                Slinf.OutType = 'V'

            self.UpdateRownTableChannels(Slinf,
                                         Row=index+offset)

        self.TblChs.setSortingEnabled(True)

    def UpdateRownTableChannels(self, Slinf, Row):

        for f, c in self.ChFields.iteritems():
            item = QtWidgets.QTableWidgetItem()
            if c[3] == 0:
                item.setFlags(item.flags() & ~Qt.ItemIsEditable)

            val = getattr(Slinf, f)
            if c[2] == 1 or c[2] == 2 or c[2] == 3:
                item.setData(Qt.DisplayRole, str(val))
            else:
                item.setData(Qt.DisplayRole, val)

            self.TblChs.setItem(Row, c[1], item)

    def ButPlotClick(self):

        Rows = self.TblChs.rowCount()

        slots = []

        for r in range(Rows):
            sl = PltSlot()

            for f, c in self.ChFields.iteritems():
                v = self.TblChs.item(r, c[1]).text()
                if f == 'GmSignal':
                    if v != '':
                        setattr(sl, f, (self.rec2, v))
                    continue

                if c[2] == 1:
                    setattr(sl, f, int(v))
                elif c[2] == 2:
                    vv = v.split(' ')
                    setattr(sl, f, float(vv[0])*pq.s)
                elif c[2] == 3:
                    setattr(sl, f, float(v))
                elif c[2] == 4:
                    vl = v.split(',')
                    print vl
                    setattr(sl, f, vl)
                elif c[2] == 5:
                    vl = v.split(',')
                    try:
                        vf = [float(i) for i in vl]
                        setattr(sl, f, vf)
                    except:
                        print 'Error in input data ', v, vl
                else:
                    setattr(sl, f, v)

            if sl.RecN == 1:
                setattr(sl, 'rec', self.rec)
            else:
                setattr(sl, 'rec', self.rec2)

#            if not sl.FiltType[0] == '':
#                sl.DispName = sl.DispName+'F'

            slots.append(sl)

        self.pltrec = PlotRecord()
        self.pltrec.CreateFig(slots)
        plt.show()

        self.LstSlots.clear()
        for i, sl in enumerate(self.pltrec.Slots):
            self.LstSlots.addItem(sl.DispName)

    def UpdateTimeControl(self):
        starts = []
        stops = []

        for sig in self.rec.Seg.analogsignals:
            starts.append(sig.t_start)
            stops.append(sig.t_stop)

        Ctrs = (self.SLTStart,
                self.SLTStop,
                self.SpnTStart,
                self.SpnAvgTimeStart,
                self.SpnAvgTimeStop,
                self.SpnPSDTimeStart,
                self.SpnPSDTimeStop)

        print stops, starts
        self.SpnTShow.setMaximum(max(stops)-min(starts))

        self.UpdateTstart(Ctrs, min(starts))
        self.UpdateTstop(Ctrs, max(stops))
        self.LblTstartMax.setText(str(max(stops)))
        self.LblStopMax.setText(str(max(stops)))

    def UpdateTstart(self, Controls, Value):
        for ctr in Controls:
            ctr.setMinimum(Value)

    def UpdateTstop(self, Controls, Value):
        for ctr in Controls:
            ctr.setMaximum(Value)

    def SpnTStartChanged(self):
        self.SLTStart.setValue(self.SpnTStart.value())
        self.SLTStop.setValue(self.SpnTStart.value()+self.SpnTShow.value())

    def SpnTShowChanged(self):
        self.SLTStop.setValue(self.SpnTStart.value()+self.SpnTShow.value())

    def SLTStartChanged(self):
        self.SpnTStart.setValue(self.SLTStart.value())
        self.SLTStop.setValue(self.SpnTStart.value()+self.SpnTShow.value())

    def SLTStopChanged(self):
        if (self.SLTStop.value() < self.SLTStart.value()):
            self.SLTStop.setValue(self.SLTStart.value()+1)
        self.SpnTShow.setValue(self.SLTStop.value()-self.SLTStart.value())
        self.UpdatePlot()

    def UpdatePlot(self):
        Tstart = self.SLTStart.value()
        Tstop = self.SLTStop.value()

        self.SpnAvgTimeStart.setValue(Tstart)
        self.SpnAvgTimeStop.setValue(Tstop)
        self.SpnPSDTimeStart.setValue(Tstart)
        self.SpnPSDTimeStop.setValue(Tstop)

        resampPoints = self.SpnResampPoints.value()
        resamp = self.ChckResamp.isChecked()

        try:
            self.pltrec.ClearAxes()
            self.pltrec.PlotChannels((Tstart*pq.s, Tstop*pq.s),
                                     ResampPoints=resampPoints,
                                     Resamp=resamp)
            self.pltrec.Fig.canvas.draw()
        except:
            print "Unexpected error:", sys.exc_info()[0]

        (EventRec, EventName) = self.GetSelectedEvent()
        if EventName:
            try:
                self.pltrec.PlotEvents((Tstart*pq.s, Tstop*pq.s),
                                       (EventRec, EventName))
            except:
                print 'Event error'
            self.pltrec.Fig.canvas.draw()

    def ButNextEventClick(self):
        EventName = self.GetSelectedEvent()

        if EventName:
            etimes = EventName[0].GetEventTimes(EventName[1]) 
            Tstart = self.SLTStart.value()
            
            ne = np.where(etimes > Tstart)
            print etimes[ne[0][1]]          
            
            tsw = self.SpnTShow.value()
            self.SLTStart.setValue(etimes[ne[0][1]].magnitude-tsw/2)
        
    def ButButPSDClick(self):
        Tstart = self.SLTStart.value()
        Tstop = self.SLTStop.value()
        nFFT = 2**self.SpnPSDViewnFFT.value()

        self.pltrec.PlotPSD((Tstart*pq.s, Tstop*pq.s), nFFT=nFFT)
        plt.show()
        self.pltrec.FigFFT.canvas.draw()

    def ButEventAvgClick(self):

        Tstart = self.SpnAvgTimeStart.value()
        Tstop = self.SpnAvgTimeStop.value()

        Estart = self.SpnAvgEventStart.value()
        Estop = self.SpnAvgEventStop.value()

        EventName = self.GetSelectedEvent()

        if EventName:
            self.pltrec.PlotEventAvg(EventName,
                                     Time=(Tstart*pq.s, Tstop*pq.s),
                                     TimeWindow=(Estart*pq.s, Estop*pq.s),
                                     OverLap=self.ChkEventOverlap.isChecked(),
                                     Std=self.ChkEventStd.isChecked(),
                                     Spect=self.ChkEventSpect.isChecked())

    def GetSelectedEvent(self):
        EventName = None
        EventRec = None

        if self.ChckEvent.isChecked():
            events = []
            for s in self.LstEvents.selectedItems():
                events.append(s.text())
            if len(events) > 0:
                EventName = events[0]
                EventRec = self.rec

            if not EventName:
                for s in self.LstEvents2.selectedItems():
                    events.append(s.text())
                if len(events) > 0:
                    EventName = events[0]
                    EventRec = self.rec2

        return EventRec, EventName

    def SaveJoinRec(self):
        RecordFile, _ = QFileDialog.getSaveFileName(self,
                                                    "Rec1 file", "",
                                                    "NixIO (*.h5)")
        print RecordFile
        if RecordFile:
            for sig in self.rec2.Seg.analogsignals:
                print sig.name
                sig.t_start = self.SpnTstart2.value()*pq.s
                self.rec.AddSignal(sig*self.rec.UnitGain)
            self.rec.SaveRecord(RecordFile)

    def SaveRec1(self):
        RecordFile, _ = QFileDialog.getSaveFileName(self,
                                                    "Rec1 file", "",
                                                    "NixIO (*.h5)")
        if RecordFile:
            self.rec.SaveRecord(RecordFile)

    def ButSaveViewClick(self):  # TODO finish save

        if self.ChkSaveRec1.isChecked():
            RecordFile, _ = QFileDialog.getSaveFileName(self,
                                                        "Rec1 file", "",
                                                        "NixIO (*.h5)")
            if RecordFile:
                out_f = neo.io.NixIO(filename=RecordFile)
                out_f.write_block(self.rec.Block)
                out_f.close()

        if self.ChkSaveRec2.isChecked():
            RecordFile, _ = QFileDialog.getSaveFileName(self,
                                                        "Rec2 file", "",
                                                        "NixIO (*.h5)")
            if RecordFile:
                out_f = neo.io.NixIO(filename=RecordFile)
                out_f.write_block(self.rec2.Block)
                out_f.close()


        if self.ChkSaveView.isChecked():
            RecordFile, _ = QFileDialog.getSaveFileName(self,
                                                        "Current View file", "",
                                                        "NixIO (*.h5)")
            if RecordFile:
                Tstart = self.SLTStart.value()*pq.s
                Tstop = self.SLTStop.value()*pq.s

                out_seg = neo.Segment(name='NewSeg')

                for sl in self.pltrec.Slots:
                    SName = sl.DispName
                    print SName
                    sig = sl.rec.Signal(SName)
                    out_seg.analogsignals.append(sig.time_slice(Tstart, Tstop))

                out_bl = neo.Block(name='NewBlock')
                out_bl.segments.append(out_seg)
                out_f = neo.io.NixIO(filename='Test_out.h5')
                out_f.write_block(out_bl)
                out_f.close()

    def ButThresClick(self):  # TODO move to PltSlot
        
        sl = self.pltrec.Slots[self.LstSlots.currentRow()]
        sig = sl.GetSignal((sl.Signal().t_start, sl.Signal().t_stop),
                           Resamp=False)

#        sign = str(self.CmbThSign.currentText())
#        print sign, type(sign)
        eve = threshold_detection(sig,
                                  threshold=float(self.TxvThVal.text())*sig.units,
                                  sign=self.CmbThSign.currentText(),
                                  RelaxTime=float(self.TxvThRTime.text())*pq.s)

        for e in eve:
            print e

        sl.rec.AddEvent(eve, self.TxvThName.text())

        if sl.RecN == 1:
            self.LstEvents.clear()
            for ch in sorted(self.rec.EventNames.keys()):
                self.LstEvents.addItem(ch)
        elif sl.RecN == 2:
            self.LstEvents2.clear()
            for ch in sorted(self.rec2.EventNames.keys()):
                self.LstEvents2.addItem(ch)

#        self.LstEvents.clear()
#        for ch in sorted(self.rec.EventNames.keys()):
#            self.LstEvents.addItem(ch)

    def ButLoadDCClick(self):
        DCFile, _ = QFileDialog.getOpenFileName(self,
                                                "DC charact file", "",
                                                "(*.h5);; (*.*)")

        if DCFile:
            self.DevDCVals, _ = FETData.LoadDataFromFile(DCFile)
            self.LblDCFile.setText(DCFile.split('/')[-1])
            self.ButViewDCClick()

    def ButViewDCClick(self):
        if self.DevDCVals:
            pltDC = FETplt.PyFETPlot()
            pltDC.AddAxes(('Ids', 'Gm', 'Rds'))
            pltDC.PlotDataCh(self.DevDCVals)
            pltDC.AddLegend()
            plt.show()
            pltDC.Fig.canvas.draw()
        else:
            self.ButLoadDCClick()

    def ButPSDSNRClick(self):
        if not self.DevDCVals:
            self.ButLoadDCClick()
            if not self.DevDCVals:
                return

        Tstart = self.SpnPSDTimeStart.value()*pq.s
        Tstop = self.SpnPSDTimeStop.value()*pq.s
        Estart = self.SpnPSDEventStart.value()*pq.s
        Estop = self.SpnPSDEventStop.value()*pq.s
        nFFT = 2**self.SpnPSDnFFT.value()

        EventName = self.GetSelectedEvent()
        if EventName:
            self.pltrec.PlotPSDSNR(EventName,
                                     TDelay=Estart,
                                     TEval=Estop,
                                     DevDCVals=self.DevDCVals,
                                     Time=(Tstart, Tstop),
                                     nFFT=nFFT)


def main():
    import argparse
    import pkg_resources

    # Add version option
    __version__ = pkg_resources.require("PyGFET")[0].version
    parser = argparse.ArgumentParser()
    parser.add_argument('--version', action='version',
                        version='%(prog)s {version}'.format(
                            version=__version__))
    parser.parse_args()

    app = QtWidgets.QApplication(sys.argv)
    w = RecordExplorer()
    w.show()
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()
